import falcon

import sms_gateway.config as config


def user_loader(token):
    if token == config.TOKEN:
        return 'user'
    else:
        return None


def get_parameter(req, param):
    param_value = req.get_param(param)
    if param_value is None or param_value == "":
        raise falcon.HTTPBadRequest(
            'Missing parameter',
            f"The parameter '{param}' has to be specified."
        )

    return param_value



import logging
import traceback

import sms_gateway.config as config


def send_sms():
    while True:
        data = config.SMS_QUEUE.get()
        try:
            sms_message = {
                'Number': data[0],
                'Text': data[1],
                'SMSC': {'Location': 1}
            }
            config.SM.SendSMS(sms_message)
            logging.info(f'Successfully sent SMS message to {data[0]}')
        except Exception:
            logging.error(f"Error while sending SMS message '{data[1]}' to {data[0]}\n{traceback.format_exc()}")



import json
import re

import falcon

import sms_gateway.config as config
from .controller_utils import get_parameter


class SMS(object):

    def on_get(self, req, resp):
        sms_messages = []

        start = True
        status = config.SM.GetSMSStatus()
        remain = status['SIMUsed'] + status['PhoneUsed'] + status['TemplatesUsed']
        while remain > 0:
            if start:
                sms = config.SM.GetNextSMS(Start=True, Folder=0)
                start = False
            else:
                sms = config.SM.GetNextSMS(Location=sms[0]['Location'], Folder=0)

            remain -= len(sms)
            sms_messages.append({'number': sms[0]['Number'], 'text': sms[0]['Text']})

        resp.body = json.dumps(sms_messages)
        resp.status = falcon.HTTP_200

    def on_post(self, req, resp):
        number = get_parameter(req, "number")
        text = get_parameter(req, "text")

        # Remove spaces from number
        number = number.replace(" ", "")

        # Check whitelist
        pattern = re.compile(config.WHITELIST)
        if not pattern.match(number):
            raise falcon.HTTPBadRequest(
                'Number not allowed',
                f"The number '{number}' is not allowed. Check your whitelist configuration."
            )

        config.SMS_QUEUE.put((number, text))

        resp.status = falcon.HTTP_202



import multiprocessing
import os

import gammu

# Variables
LOGLEVEL = os.environ.get('SG_LOGLEVEL')
PORT = os.environ.get('SG_PORT')
PIN = os.environ.get('SG_PIN')
TOKEN = os.environ.get('SG_TOKEN')
DEVICE = os.environ.get('SG_DEVICE')
WHITELIST = os.environ.get('SG_WHITELIST')

# SMS Queue
SMS_QUEUE = multiprocessing.Queue()

# Gammu State Machine
SM = gammu.StateMachine()



import argparse
import logging
import secrets
import threading
import time
from wsgiref.simple_server import make_server

import falcon
from falcon_auth import FalconAuthMiddleware, TokenAuthBackend

import sms_gateway.config as config
from .background_threads.sms_sender import send_sms
from .controllers.controller_utils import user_loader
from .controllers.sms import SMS

auth_backend = TokenAuthBackend(user_loader)
auth_middleware = FalconAuthMiddleware(auth_backend)

api = falcon.API(middleware=[auth_middleware])

sms = SMS()
api.add_route('/sms', sms)

parser = argparse.ArgumentParser()
parser.add_argument('--loglevel', type=str, help="Sets the log level.", default='INFO')
parser.add_argument('--pin', type=str, help="Sets the PIN used to unlock the modem.", default=None)
parser.add_argument('--port', type=int, help="Sets the port the web app is listening on.", default=8000)
parser.add_argument('--token', type=str, help="Sets the authentication token.", default=None)
parser.add_argument('--device', type=str, help="Sets the device to use with gammu.", default='/dev/modem')
parser.add_argument('--whitelist', type=str, help="Sets a whitelist for the recipient numbers.", default='.+')


def main():
    parse_options(parser.parse_args())
    unlock_modem()

    # Start background thread that sends SMS
    sms_sender = threading.Thread(name="SMSSender", target=send_sms)
    sms_sender.setDaemon(True)
    sms_sender.start()

    # Start web app
    with make_server('', config.PORT, api) as httpd:
        logging.info(f'Serving on port {config.PORT}...')
        httpd.serve_forever()


def parse_options(options):
    # Log level
    if config.LOGLEVEL is None:
        config.LOGLEVEL = options.loglevel
    config.LOGLEVEL = config.LOGLEVEL.upper()
    logging.basicConfig(level=config.LOGLEVEL, format='%(asctime)s [%(levelname)s] %(message)s')

    # PIN
    if config.PIN is None:
        config.PIN = options.pin
    if config.PIN is None or len(config.PIN) == 0:
        raise Exception('PIN is required to unlock the modem')

    # Port
    if config.PORT is None:
        config.PORT = options.port
    config.PORT = int(config.PORT)

    # Token
    if config.TOKEN is None:
        config.TOKEN = options.token
    if config.TOKEN is None or config.TOKEN == '':
        config.TOKEN = secrets.token_urlsafe(20)
        logging.warning(f"No token set - using auto-generated token '{config.TOKEN}' instead")

    # Device
    if config.DEVICE is None:
        config.DEVICE = options.device

    # Whitelist
    if config.WHITELIST is None:
        config.WHITELIST = options.whitelist


def unlock_modem():
    # Initialize
    config.SM.SetConfig(0, {'Device': config.DEVICE, 'Connection': 'at'})
    config.SM.Init()

    # Unlock
    status = config.SM.GetSecurityStatus()
    if status is None:
        logging.info('Modem is already unlocked')
    elif config.SM.GetSecurityStatus() == 'PIN':
        logging.info('Unlocking modem...')
        config.SM.EnterSecurityCode('PIN', config.PIN)
        time.sleep(10)
    else:
        raise Exception(f"Unexpected security status '{status}'")


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages
import io


def readme():
    with io.open('README.md', encoding='utf-8') as f:
        return f.read()


setup(
    name='sms-gateway',
    description='Web App Serving as an SMS Gateway based on Gammu',
    long_description=readme(),
    long_description_content_type='text/markdown',
    url='https://github.com/joneug/sms-gateway',
    author='joneug',
    packages=find_packages('.'),
    keywords=['SMS', 'gateway', 'gammu'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Environment :: Console",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires=' >= 3.5',
    version='0.1.9',
    license='MIT',
    install_requires=[
        "falcon >= 2.0.0",
        "falcon_auth >= 1.1.0",
        "python-gammu >= 3.1"
    ],
    entry_points={
        'console_scripts': [
            'sms-gateway = sms_gateway.__main__:main'
        ]
    },
    project_urls={
        'Documentation': 'https://github.com/joneug/sms-gateway',
        'Source': 'https://github.com/joneug/sms-gateway',
        'Tracker': 'https://github.com/joneug/sms-gateway/issues',
    }
)
